@@ -72,7 +72,7 @@ void print_usage(const string& progname)
 {
   cout << endl;
   cout << "Usage: fslcorrecthd <input> <output>" << endl;
-  cout << "       Note that fslcorrecthd only works uncompressed NIFTI or ANALYZE files" << endl;
+  cout << "       Note that fslcorrecthd only operates on uncompressed NIFTI or ANALYZE files" << endl;
 }
 
 int main(int argc,char *argv[])
@@ -110,10 +110,11 @@ int main(int argc,char *argv[])
 
  ifstream input_file;
  ofstream output_file;
- char *temp,*outputName; 
+ char *temp,*outputName,*inputName; 
  FslGetHdrImgNames(argv[2],fslio,&temp,&outputName);
+ FslGetHdrImgNames(argv[1],fslio,&temp,&inputName);
  char byte[1];
- input_file.open(argv[1],ios::in | ios :: binary);
+ input_file.open(inputName,ios::in | ios :: binary);
  output_file.open(outputName,ofstream::out | ofstream::binary);
 
  for(int i=1;i<=minft;i++) //Write Header
