@@ -96,6 +96,7 @@ class pvmfitOptions {
   Option<string> bvalsfile;
   Option<int>    nfibres;
   Option<int>    modelnum;
+  Option<bool>   include_f0;
   Option<bool>   cnonlinear;
 
   bool parse_command_line(int argc, char** argv);
@@ -144,10 +145,13 @@ class pvmfitOptions {
 	     string("number of fibres to fit - default=1"),
 	     false, requires_argument), 
    modelnum(string("--model"), 1,
-	     string("1:monoexponential;2:non mono-exponential"),
+	     string("\t1:monoexponential;2:non mono-exponential"),
 	     false, requires_argument), 
+   include_f0(string("--f0"),false, 
+	      string("\tInclude noise floor in the model (default=false)"),
+	      false,no_argument),
    cnonlinear(string("--cnonlinear"),false, 
-	      string("Apply constrained nonlinear optimization on the diffusivity, volume fractions and their sum"),
+	      string("Apply constrained nonlinear optimization on diffusivity, volume fractions (model1)"),
 	      false,no_argument),
    options("pvmfit", "pvmfit -k <datafile> -m <maskfile> -r <bvecsfile> -b <bvalsfile> [-n 2]\n")
    {
@@ -163,6 +167,7 @@ class pvmfitOptions {
        options.add(bvalsfile);
        options.add(nfibres);
        options.add(modelnum);
+       options.add(include_f0);
        options.add(cnonlinear);
      }
      catch(X_OptionError& e) {
