@@ -73,8 +73,8 @@
 
 using namespace Utilities;
 
-string title="vecreg (Version 1.0)\nVector Affine/non linear Tranformation with Orientation Preservation";
-string examples="vecreg -i <input4Dvector> -o <output4D> -t <transformation>";
+string title="vecreg (Version 1.1)\nVector Affine/NonLinear Tranformation with Orientation Preservation";
+string examples="vecreg -i <input4D> -o <output4D> -r <refvol> -t <transform>";
 
 Option<bool> verbose(string("-v,--verbose"),false,
 		       string("switch on diagnostic messages"),
@@ -82,30 +82,36 @@ Option<bool> verbose(string("-v,--verbose"),false,
 Option<bool> help(string("-h,--help"),false,
 		       string("display this message"),
 		       false,no_argument);
-Option<string> ivector(string("-i,--input"),string(""),
-		       string("filename of input vector"),
-		       false,requires_argument);
-Option<string> itensor(string("--tensor"),string(""),
-		       string("full tensor"),
-		       false,requires_argument);
-Option<string> ovector(string("-o,--output"),string(""),
-		       string("filename of output registered vector"),
+Option<string> infilename(string("-i,--input"),string(""),
+		       string("filename for input vector or tensor field"),
+		       true,requires_argument);
+Option<string> outfilename(string("-o,--output"),string(""),
+		       string("filename for output registered vector or tensor field"),
 		       true,requires_argument);
 Option<string> ref(string("-r,--ref"),string(""),
-		       string("filename of reference (target) volume"),
+		       string("filename for reference (target) volume"),
 		       true,requires_argument);
 Option<string> matrix(string("-t,--affine"),string(""),
-		       string("filename of affine transformation matrix"),
+		       string("filename for affine transformation matrix"),
 		       false,requires_argument);
 Option<string> warp(string("-w,--warpfield"),string(""),
-		       string("filename of 4D warp field for nonlinear registration"),
+		       string("filename for 4D warp field for nonlinear registration"),
+		       false,requires_argument);
+Option<string> matrix2(string("--rotmat"),string(""),
+		       string("filename for secondary affine matrix \n\t\t\tif set, this will be used for the rotation of the vector/tensor field "),
+		       false,requires_argument);
+Option<string> warp2(string("--rotwarp"),string(""),
+		       string("filename for secondary warp field \n\t\t\tif set, this will be used for the rotation of the vector/tensor field "),
 		       false,requires_argument);
 Option<string> interpmethod(string("--interp"),"",
-		       string("interpolation method : nearestneighbour, trilinear (default) or sinc"),
+		       string("interpolation method : nearestneighbour, trilinear (default), sinc or spline"),
 		       false,requires_argument);
 Option<string> maskfile(string("-m,--mask"),string(""),
 		       string("brain mask in input space"),
 		       false,requires_argument);
+Option<string> omaskfile(string("--refmask"),string(""),
+			 string("brain mask in output space (useful for speed up of nonlinear reg)"),
+			 false,requires_argument);
 ////////////////////////////////////////////////////////
 
 ReturnMatrix rodrigues(const float& angle,ColumnVector& w){
@@ -197,80 +203,6 @@ ReturnMatrix ppd(const Matrix& F,ColumnVector& e1){
   return R;
 }
 
-
-void vecreg_aff(const volume4D<float>& tens,
-		volume4D<float>& oV1,
-		const volume<float>& refvol,
-		const Matrix& M,
-		const volume<float>& mask){
-
-  Matrix iM(4,4),R(3,3);
-  iM=M.i();
-  // extract rotation matrix from M
-  Matrix F(3,3),u(3,3),v(3,3);
-  DiagonalMatrix d(3);
-  F=M.SubMatrix(1,3,1,3);
-  SVD(F*F.t(),d,u,v);
-  R=(u*sqrt(d)*v.t()).i()*F;
-
-  ColumnVector seeddim(3),targetdim(3);
-  seeddim << tens.xdim() << tens.ydim() << tens.zdim();
-  targetdim  << refvol.xdim() << refvol.ydim() << refvol.zdim();
-  SymmetricMatrix Tens(3);
-  ColumnVector X_seed(3),X_target(3); 
-  ColumnVector V_seed(3),V_target(3);
-  for(int z=0;z<oV1.zsize();z++)
-    for(int y=0;y<oV1.ysize();y++)
-      for(int x=0;x<oV1.xsize();x++){
-
-	// compute seed coordinates
-	X_target << x << y << z;
-	X_seed=vox_to_vox(X_target,targetdim,seeddim,iM);
-	
-	if(mask((int)X_seed(1),(int)X_seed(2),(int)X_seed(3))==0){
-	  continue;
-	}
-
-	 // compute interpolated tensor
-	 Tens.Row(1) << tens[0].interpolate(X_seed(1),X_seed(2),X_seed(3));
-	 Tens.Row(2) << tens[1].interpolate(X_seed(1),X_seed(2),X_seed(3))
-		     << tens[3].interpolate(X_seed(1),X_seed(2),X_seed(3));
-	 Tens.Row(3) << tens[2].interpolate(X_seed(1),X_seed(2),X_seed(3))
-		     << tens[4].interpolate(X_seed(1),X_seed(2),X_seed(3))
-		     << tens[5].interpolate(X_seed(1),X_seed(2),X_seed(3));
-
-
-	if(ivector.set()){
-	  // compute first eigenvector
-	  EigenValues(Tens,d,v);
-	  V_seed = v.Column(3);
-	  
-	  // rotate vector
-	  V_target=R*V_seed;
-	  if(V_target.MaximumAbsoluteValue()>0)
-	    V_target/=sqrt(V_target.SumSquare());
-
-	  oV1(x,y,z,0)=V_target(1);
-	  oV1(x,y,z,1)=V_target(2);
-	  oV1(x,y,z,2)=V_target(3);
-	}
-	
-	// create tensor
-	if(ivector.unset()){
-	  Tens << R*Tens*R.t();
-	  
-	  oV1(x,y,z,0)=Tens(1,1);
-	  oV1(x,y,z,1)=Tens(2,1);
-	  oV1(x,y,z,2)=Tens(3,1);
-	  oV1(x,y,z,3)=Tens(2,2);
-	  oV1(x,y,z,4)=Tens(3,2);
-	  oV1(x,y,z,5)=Tens(3,3);
-	  
-	}
-	
-      }
-  
-}
 void sjgradient(const volume<float>& im,volume4D<float>& grad){
   
   grad.reinitialize(im.xsize(),im.ysize(),im.zsize(),3);
@@ -299,6 +231,126 @@ void sjgradient(const volume<float>& im,volume4D<float>& grad){
 
 }
 
+void vecreg_aff(const volume4D<float>& tens,
+		volume4D<float>& oV1,
+		const volume<float>& refvol,
+		const Matrix& M,
+		const volume<float>& mask){
+
+  Matrix iM(4,4);
+  iM=M.i();
+
+  /////////////////////////////////////////////////////////////////////////
+  // Where we define a potential affine transfo for the rotation
+  Matrix M2;
+  if(matrix2.value()!="")
+    M2 = read_ascii_matrix(matrix2.value());
+  // Where we define a potential warp field transfo for the rotation
+  volume4D<float> warpvol2;
+  volume4D<float> jx,jy,jz;
+  Matrix Jw(3,3),I(3,3);I<<1<<0<<0<<0<<1<<0<<0<<0<<1;
+  if(warp2.value()!=""){
+    FnirtFileReader ffr2(warp2.value());
+    warpvol2=ffr2.FieldAsNewimageVolume4D(true);
+    sjgradient(warpvol2[0],jx);
+    sjgradient(warpvol2[1],jy);
+    sjgradient(warpvol2[2],jz);
+  }
+  /////////////////////////////////////////////////////////////////////////
+  volume<float> omask;
+  if(omaskfile.value()!="")
+    read_volume(omask,omaskfile.value());
+
+
+  // extract rotation matrix from M
+  Matrix F(3,3),R(3,3),u(3,3),v(3,3);
+  DiagonalMatrix d(3);
+  if(matrix2.value()=="")
+    F=M.SubMatrix(1,3,1,3);
+  else
+    F=M2.SubMatrix(1,3,1,3);
+  SVD(F*F.t(),d,u,v);
+  R=(u*sqrt(d)*v.t()).i()*F;
+
+  ColumnVector seeddim(3),targetdim(3);
+  seeddim << tens.xdim() << tens.ydim() << tens.zdim();
+  targetdim  << refvol.xdim() << refvol.ydim() << refvol.zdim();
+  SymmetricMatrix Tens(3);
+  ColumnVector X_seed(3),X_target(3); 
+  ColumnVector V_seed(3),V_target(3);
+  for(int z=0;z<oV1.zsize();z++)
+    for(int y=0;y<oV1.ysize();y++)
+      for(int x=0;x<oV1.xsize();x++){
+	if(omaskfile.value()!="")
+	  if(omask(x,y,z)==0)
+	    continue;
+
+	// compute seed coordinates
+	X_target << x << y << z;
+	X_seed=vox_to_vox(X_target,targetdim,seeddim,iM);
+	
+	if(mask((int)round(float(X_seed(1))),(int)round(float(X_seed(2))),(int)round(float(X_seed(3))))==0)
+	  continue;
+	
+
+	 // compute interpolated tensor
+	 Tens.Row(1) << tens[0].interpolate(X_seed(1),X_seed(2),X_seed(3));
+	 Tens.Row(2) << tens[1].interpolate(X_seed(1),X_seed(2),X_seed(3))
+		     << tens[3].interpolate(X_seed(1),X_seed(2),X_seed(3));
+	 Tens.Row(3) << tens[2].interpolate(X_seed(1),X_seed(2),X_seed(3))
+		     << tens[4].interpolate(X_seed(1),X_seed(2),X_seed(3))
+		     << tens[5].interpolate(X_seed(1),X_seed(2),X_seed(3));
+
+
+
+	 if(warp2.value()!=""){
+	   // Local Jacobian of the backward warpfield
+	   Jw <<   jx(x,y,z,0) <<  jx(x,y,z,1) << jx(x,y,z,2)
+	      <<   jy(x,y,z,0) <<  jy(x,y,z,1) << jy(x,y,z,2)
+	      <<   jz(x,y,z,0) <<  jz(x,y,z,1) << jz(x,y,z,2); 
+	   // compute local forward affine transformation	
+	   F = (I + Jw).i();
+	 }
+	 
+	 if(oV1.tsize()==3){ // case where input is a vector
+	   // compute first eigenvector
+	   EigenValues(Tens,d,v);
+	   V_seed = v.Column(3);
+	   
+	   // rotate vector
+	   V_target=F*V_seed;
+	   if(V_target.MaximumAbsoluteValue()>0)
+	     V_target/=sqrt(V_target.SumSquare());
+	   
+	   oV1(x,y,z,0)=V_target(1);
+	   oV1(x,y,z,1)=V_target(2);
+	   oV1(x,y,z,2)=V_target(3);
+	 }
+	
+	 // create tensor
+	 if(oV1.tsize()==6){
+	   if(warp2.value()!=""){
+	     EigenValues(Tens,d,v);
+	     R=ppd(F,v.Column(3),v.Column(2));
+	     Tens << R*Tens*R.t();
+	   }
+	   else{
+	     Tens << R*Tens*R.t();
+	   }
+	   
+	   oV1(x,y,z,0)=Tens(1,1);
+	   oV1(x,y,z,1)=Tens(2,1);
+	   oV1(x,y,z,2)=Tens(3,1);
+	   oV1(x,y,z,3)=Tens(2,2);
+	   oV1(x,y,z,4)=Tens(3,2);
+	   oV1(x,y,z,5)=Tens(3,3);
+	   
+	 }
+	
+      }
+  
+}
+
 
 void vecreg_nonlin(const volume4D<float>& tens,volume4D<float>& oV1,
 		   const volume<float>& refvol,volume4D<float>& warpvol,
@@ -306,46 +358,63 @@ void vecreg_nonlin(const volume4D<float>& tens,volume4D<float>& oV1,
 
   ColumnVector X_seed(3),X_target(3);
   
-  //float dxx=tens.xdim(),dyy=tens.ydim(),dzz=tens.zdim();
-  //float dx=oV1.xdim(),dy=oV1.ydim(),dz=oV1.zdim();
-  //float nxx=(float)tens.xsize()/2.0,nyy=(float)tens.ysize()/2.0,nzz=(float)tens.zsize()/2.0;
-  //float nx=(float)oV1.xsize()/2.0,ny=(float)oV1.ysize()/2.0,nz=(float)oV1.zsize()/2.0;
-
   
   // read warp field created by Jesper
   FnirtFileReader ffr(warp.value());
   warpvol=ffr.FieldAsNewimageVolume4D(true);
  
-
-  // compute transformation jacobian
-  volume4D<float> jx(mask.xsize(),mask.ysize(),mask.zsize(),3);
-  volume4D<float> jy(mask.xsize(),mask.ysize(),mask.zsize(),3);
-  volume4D<float> jz(mask.xsize(),mask.ysize(),mask.zsize(),3);
-  sjgradient(warpvol[0],jx);
-  sjgradient(warpvol[1],jy);
-  sjgradient(warpvol[2],jz);
-
+  Matrix F(3,3),u(3,3),v(3,3);
+  DiagonalMatrix d(3);
+    
+  /////////////////////////////////////////////////////////////////////////
+  // Where we define a potential affine transfo for the rotation
+  Matrix M2;
+  if(matrix2.value()!=""){
+    M2 = read_ascii_matrix(matrix2.value());
+    // extract rotation matrix from M
+    F=M2.SubMatrix(1,3,1,3);
+    SVD(F*F.t(),d,u,v);
+    F=(u*sqrt(d)*v.t()).i()*F;
+  }
+  // Where we define a potential warp field transfo for the rotation
+  volume4D<float> warpvol2;
+  volume4D<float> jx,jy,jz;
+  if(warp2.value()!=""){
+    FnirtFileReader ffr2(warp2.value());
+    warpvol2=ffr2.FieldAsNewimageVolume4D(true);
+    sjgradient(warpvol2[0],jx);
+    sjgradient(warpvol2[1],jy);
+    sjgradient(warpvol2[2],jz);
+  }
+  else{
+    sjgradient(warpvol[0],jx);
+    sjgradient(warpvol[1],jy);
+    sjgradient(warpvol[2],jz);
+  }
+  /////////////////////////////////////////////////////////////////////////
+  volume<float> omask;
+  if(omaskfile.value()!="")
+    read_volume(omask,omaskfile.value());
 
   ColumnVector V_seed(3),V_target(3);
   ColumnVector V1_seed(3),V2_seed(3);
   ColumnVector V1_target(3),V2_target(3),V3_target(3);
   Matrix R(3,3),I(3,3);I<<1<<0<<0<<0<<1<<0<<0<<0<<1;
-  Matrix F(3,3),Jw(3,3),u(3,3),v(3,3);
-  DiagonalMatrix d(3);  
+  Matrix Jw(3,3);
   SymmetricMatrix Tens(3);
   for(int z=0;z<oV1.zsize();z++)
     for(int y=0;y<oV1.ysize();y++)
        for(int x=0;x<oV1.xsize();x++){
-	 
-
+	 if(omaskfile.value()!="")
+	   if(omask(x,y,z)==0)
+	     continue;
 
 	 X_target << x << y << z;
 	 X_seed = NewimageCoord2NewimageCoord(warpvol,false,oV1[0],mask,X_target);
 
 
-	 if(mask((int)X_seed(1),(int)X_seed(2),(int)X_seed(3))==0){
-	   continue;
-	 }
+	if(mask((int)round(float(X_seed(1))),(int)round(float(X_seed(2))),(int)round(float(X_seed(3))))==0)
+	  continue;
 	
 	 // compute interpolated tensor
 	 Tens.Row(1) << tens[0].interpolate(X_seed(1),X_seed(2),X_seed(3));
@@ -355,25 +424,23 @@ void vecreg_nonlin(const volume4D<float>& tens,volume4D<float>& oV1,
 		     << tens[4].interpolate(X_seed(1),X_seed(2),X_seed(3))
 		     << tens[5].interpolate(X_seed(1),X_seed(2),X_seed(3));
 
-	 // Local Jacobian of the backward warpfield
-	 Jw <<   jx(x,y,z,0) <<  jx(x,y,z,1) << jx(x,y,z,2)
-	    <<   jy(x,y,z,0) <<  jy(x,y,z,1) << jy(x,y,z,2)
-	    <<   jz(x,y,z,0) <<  jz(x,y,z,1) << jz(x,y,z,2);
-	 
-	 // compute local forward affine transformation	
-	 F = (I + Jw).i();
-	 
-
-	 if(ivector.set()){
-	   // reorient according to affine reorientation scheme
-	   //SVD(F*F.t(),d,u,v);
-	   //R=(u*sqrt(d)*v.t()).i()*F;
+	 // F will not change if matrix2 is set
+	 if(matrix2.value()==""){
+	   // Local Jacobian of the backward warpfield
+	   Jw <<   jx(x,y,z,0) <<  jx(x,y,z,1) << jx(x,y,z,2)
+	      <<   jy(x,y,z,0) <<  jy(x,y,z,1) << jy(x,y,z,2)
+	      <<   jz(x,y,z,0) <<  jz(x,y,z,1) << jz(x,y,z,2); 
+	   // compute local forward affine transformation	
+	   F = (I + Jw).i();
+	 }
 
+	 if(oV1.tsize()==3){// case where input is a vector
 	   // compute first eigenvector
 	   EigenValues(Tens,d,v);
 	   V_seed = v.Column(3);
 	 
 	   V_target=F*V_seed;
+
 	   if(V_target.MaximumAbsoluteValue()>0)
 	     V_target/=sqrt(V_target.SumSquare());
 
@@ -382,13 +449,16 @@ void vecreg_nonlin(const volume4D<float>& tens,volume4D<float>& oV1,
 	   oV1(x,y,z,2)=V_target(3);
 	 }
 	 // create tensor
-	 if(ivector.unset()){
-	   //SVD(F*F.t(),d,u,v);
-	   //R=(u*sqrt(d)*v.t()).i()*F;
+	 if(oV1.tsize()==6){
 
-	   EigenValues(Tens,d,v);
-	   R=ppd(F,v.Column(3),v.Column(2));
-	   Tens << R*Tens*R.t();
+	   if(matrix2.value()==""){
+	     EigenValues(Tens,d,v);
+	     R=ppd(F,v.Column(3),v.Column(2));
+	     Tens << R*Tens*R.t();
+	   }
+	   else{
+	     Tens << F*Tens*F.t();
+	   }
 
 	   oV1(x,y,z,0)=Tens(1,1);
 	   oV1(x,y,z,1)=Tens(2,1);
@@ -411,27 +481,20 @@ int do_vecreg(){
   volume4D<float> ivol,warpvol;
   volume<float> refvol,mask;
   Matrix Aff(4,4);
-  volumeinfo vinfo;
 
   if((matrix.set())){
     Aff = read_ascii_matrix(matrix.value());
   }
-  if((warp.set())){
-    if(verbose.value()) cerr << "Loading warpfield" << endl;
-    read_volume4D(warpvol,warp.value());
-  }
+  //if((warp.set())){
+  //if(verbose.value()) cerr << "Loading warpfield" << endl;
+  //read_volume4D(warpvol,warp.value());
+  //}
   if(verbose.value()) cerr << "Loading volumes" << endl;
-  if(ivector.set())
-    read_volume4D(ivol,ivector.value());
-  else
-    read_volume4D(ivol,itensor.value());
-  read_volume(refvol,ref.value(),vinfo);
+  read_volume4D(ivol,infilename.value());
+  read_volume(refvol,ref.value());
 
   volume4D<float> ovol;
-  if(ivector.set())
-    ovol.reinitialize(refvol.xsize(),refvol.ysize(),refvol.zsize(),3);
-  else
-    ovol.reinitialize(refvol.xsize(),refvol.ysize(),refvol.zsize(),6);
+  ovol.reinitialize(refvol.xsize(),refvol.ysize(),refvol.zsize(),ivol.tsize());
   copybasicproperties(refvol,ovol);
 
   // set interpolation method
@@ -439,6 +502,8 @@ int do_vecreg(){
     ivol.setinterpolationmethod(nearestneighbour);
   else if(interpmethod.value()=="sinc")
     ivol.setinterpolationmethod(sinc);
+  else if(interpmethod.value()=="spline")
+    ivol.setinterpolationmethod(spline);
   else
     ivol.setinterpolationmethod(trilinear);
 
@@ -450,12 +515,10 @@ int do_vecreg(){
     for(int z=0;z<mask.zsize();z++)
       for(int y=0;y<mask.ysize();y++)
 	for(int x=0;x<mask.xsize();x++){
-	  if(ivol(x,y,z,0)*ivol(x,y,z,0)
-	     +ivol(x,y,z,1)*ivol(x,y,z,1)
-	     +ivol(x,y,z,2)*ivol(x,y,z,2) != 0)
-	    mask(x,y,z) = 1;
-	  else
+	  if(abs(ivol(x,y,z,0))==0 && abs(ivol(x,y,z,1))==0 && abs(ivol(x,y,z,2))==0)
 	    mask(x,y,z) = 0;
+	  else
+	    mask(x,y,z) = 1;
 	}
   }
 
@@ -463,7 +526,7 @@ int do_vecreg(){
   // tensor for interpolation
   volume4D<float> tens(ivol.xsize(),ivol.ysize(),ivol.zsize(),6);
   copybasicproperties(ivol,tens);
-  if(ivector.set())
+  if(ivol.tsize()==3)
     for(int z=0;z<ivol.zsize();z++) 
       for(int y=0;y<ivol.ysize();y++)  
 	for(int x=0;x<ivol.xsize();x++){
@@ -489,7 +552,8 @@ int do_vecreg(){
   }
   //cout<<"elapsed time:"<<time(NULL)-_time<<" sec"<<endl;
 
-  save_volume4D(ovol,ovector.value());
+  ovol.setDisplayMaximumMinimum(ivol.max(),ivol.min());
+  save_volume4D(ovol,outfilename.value());
 
   return 0;
 
@@ -504,14 +568,16 @@ int main(int argc,char *argv[]){
   try{
     options.add(verbose);
     options.add(help);
-    options.add(ivector);
-    options.add(itensor);
-    options.add(ovector);
+    options.add(infilename);
+    options.add(outfilename);
     options.add(ref);
     options.add(matrix);
     options.add(warp);
+    options.add(matrix2);
+    options.add(warp2);
     options.add(interpmethod);
     options.add(maskfile);
+    options.add(omaskfile);
 
     options.parse_command_line(argc,argv);
 
@@ -521,32 +587,24 @@ int main(int argc,char *argv[]){
       exit(EXIT_FAILURE);
     }
     if( (matrix.set()) && (warp.set()) ){
-      options.usage();
       cerr << endl
 	   << "Cannot specify both --affine AND --warpfield"
 	   << endl << endl;
       exit(EXIT_FAILURE);
     }
     if( (matrix.unset()) && (warp.unset()) ){
-      options.usage();
       cerr << endl
 	   << "Please Specify either --affine OR --warpfield"
 	   << endl << endl;
       exit(EXIT_FAILURE);
     }
-    if(ivector.unset()){
-      if(itensor.unset()){
-	cerr << endl;
-	cerr << "Please entre either an input vector or a tensor" << endl << endl;
-	exit(EXIT_FAILURE);
-      }
-    }
-    if(ivector.set()){
-      if(itensor.set()){
-	cerr << endl;
-	cerr << "Warning: warping the input vector, and ignoring the tensor" << endl << endl;
-      }
+    if( (warp2.set()) && (matrix2.set()) ){
+      cerr << endl
+	   << "Cannot Specify both --rotaff AND --rotwarp"
+	   << endl << endl;
+      exit(EXIT_FAILURE);
     }
+    
   }
   catch(X_OptionError& e) {
     options.usage();
