@@ -6,18 +6,27 @@ function [m] = read_pulse(fname)
 %
 %  See also: write_pulse (for binary matrices), load (for ascii matrices)
 
+if (nargin<1),
+  disp('??? Error using ==> read_pulse');
+  disp('Not enough input arguments.');
+  disp(' ');
+  return
+end
+
+magicnumber=42;
+
 % open file in big-endian
 endian='b';
 fid=fopen(fname,'r','b');
 testval = fread(fid,1,'uint32');
 % check if this gives the correct magic number
-if (testval~=42),
+if ((testval~=magicnumber) && (testval~=(magicnumber+1))),
   fclose(fid);
   % otherwise try little-endian
   fid=fopen(fname,'r','l');
   endian='l';
   testval = fread(fid,1,'uint32');
-  if (testval~=42),
+  if ((testval~=magicnumber) && (testval~=(magicnumber+1))),
     disp('Can not read this file format');
     return;
   end
@@ -28,8 +37,18 @@ end
 	% read the number of rows and columns
   nrows=fread(fid,1,'uint32');
   ncols=fread(fid,1,'uint32');
-  m=fread(fid,nrows*ncols,'double');
-  m=reshape(m,nrows,ncols);
-fclose(fid);
+  if (testval==magicnumber),
+    m=fread(fid,nrows*ncols,'double');
+    m=reshape(m,nrows,ncols);
+  end
+  if (testval==(magicnumber+1)),
+    time=fread(fid,nrows,'double');
+    mvals=fread(fid,nrows*(ncols-1),'float');
+    mvals=reshape(mvals,nrows,ncols-1);
+    m=zeros(nrows,ncols);
+    m(:,1)=time;
+    m(:,2:end)=mvals;
+  end
+  fclose(fid);
 return;
 
